/*
	This program recursivly searches a tree of mp3 file and outputs an index 
	based on id3 tags. See the script listmaker for example usage.

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dirent.h>
#include <fnmatch.h>
#include <sys/stat.h>

#define SONG_TITLE 1
#define ARTIST     2
#define ALBUM      3

int scan_directory (char *root, char *subdir, char *pattern, int tag_type);

void process_file(char *path, char *filename, int tag_type);

int main(int argc, char *argv[]) {
    int count;
    char search[512];
    char dir[512]; 
    int tag_type;
    
    if (argc!=4) {
	printf("Error: wrong number of arguments\n");  
	return 0;
    }	
    else {	
	strcpy(dir, argv[1]);
        strcpy(search, argv[2]);
	tag_type = atoi(argv[3]);
	//printf("searching %s %s\n", dir, search); 
	count = scan_directory(dir,"/", search, tag_type);

	return 1;
    }
}

/* this function is a modified mod_playlist_loaddir_scan() */ 
/* from mod_playlist.c in the irmp3 package */
int scan_directory (char *root, char *subdir, char *pattern, int tag_type) {

    DIR *dir;
    struct dirent *dirent;
    struct stat st;
    char buf[512], dirs[100][512];
    int count, dircount, i;

 
    snprintf(buf, sizeof(buf)-1, "%s%s", root, subdir);                                
    dir = opendir(buf);
    if (!dir) {
	printf("error: unable to read '%s'!\n", buf);
		return 0;
    }
    count = 0;
    dircount = 0;
    while (1) {
        dirent = readdir(dir);
        if (!dirent)
            break;
        if (!strcmp(dirent->d_name, ".") || !strcmp(dirent->d_name, ".."))
            continue;
        snprintf(buf, sizeof(buf)-1, "%s%s%s", root, subdir, dirent->d_name);
        if (stat(buf, &st)) {
            printf("scan_directory(): cannot stat '%s', skipping!\n", buf);
            continue;
        }
        if (S_ISDIR(st.st_mode)) {
            strcpy(dirs[dircount], dirent->d_name);
            dircount++;
        }
        // Add song if is matches our search string
        snprintf(buf, sizeof(buf)-1, "%s%s", subdir, dirent->d_name);
        if (!fnmatch(pattern, buf, FNM_NOESCAPE)) {
            //snprintf(buf, sizeof(buf)-1, "%s%s%s", root, subdir, dirent->d_name);
            snprintf(buf, sizeof(buf)-1, "%s%s", root, subdir);
	    count++;
	    process_file(buf, dirent->d_name, tag_type);
        }
    }
    closedir(dir);

    // process subdirs
    for (i=0; i<dircount; i++) {
        snprintf(buf, sizeof(buf)-1, "%s%s/", subdir, dirs[i]);
        count += scan_directory(root, buf, pattern, tag_type);
    }

    return count;
}

void terminate(char *buff) {
    int count;
    
    /* terminate the string */
    buff[30] = '\0';
    count = 30;
    do {
	count--;
	if (isspace(buff[count]))
	    buff[count] = '\0';
	else
	    count = 0;
    } while (count>1);
    return;
}

void process_file(char *path, char *filename, int tag_type) {
    FILE *file;
    char buff[31], title[31], artist[31], album[31];
    char *source_str;    
    char fullname[512];
    char search_str[31];
    int count, count2, length;
      
    snprintf(fullname, sizeof(fullname)-1, "%s%s", path, filename);
    if ((file = fopen (fullname, "rb")) == NULL) {
	fprintf (stderr, "Could not access [%s]\n", filename);
	fclose (file);
	return;
     }
    fseek(file, -128, SEEK_END);
    fread(buff, 1, 3, file);
    if (strncmp(buff, "TAG", 3) != 0 ) {  
	/* no id3 tag in file */
	fclose(file);
        return;
    }
    fread(title, 1, 30, file);
    fread(artist, 1, 30, file);
    fread(album, 1, 30, file);
    terminate(title);
    terminate(artist);
    terminate(album);
            
    switch (tag_type) {
	case SONG_TITLE : source_str = title;
	    break;
	case ARTIST 	: source_str = artist;
	    break;
	case ALBUM 	: source_str = album;
	    break;
    } 
    
    /*convert to lower case and remove spaces etc */
    length = strlen(source_str);

    count2 = 0;
    for (count=0; count<length; count++) {
	if (isalpha(source_str[count])) {
	    search_str[count2] = tolower(source_str[count]);
	    count2++;
	}    
    }
    search_str[count2] = '\0';
    if (strcmp(search_str, "")) {
	if (tag_type > 1) {
	    if (strncmp(search_str, "the", 3) == 0)
		printf("%s\t%s\t%s - %s\t\n", search_str+3, path, artist, album);
	    else
		printf("%s\t%s\t%s - %s\t\n", search_str, path, artist, album);
	}
	else
	    printf("%s\t%s\t%s - %s\t\n", search_str, fullname, title, artist);
    }	
    fclose(file);
}

